import { LanguagePlugin, ServiceEnvironment, LanguageServicePlugin } from '@volar/language-service';
import { SnapshotDocument } from '@volar/snapshot-document';
import type { TextDocuments } from 'vscode-languageserver';
import * as vscode from 'vscode-languageserver';
import { InitializationOptions, ProjectContext, ServerProjectProvider, ServerProjectProviderFactory, ServerRuntimeEnvironment } from './types.js';
import { type UriMap } from './utils/uriMap.js';
export interface ServerContext {
    initializeParams: Omit<vscode.InitializeParams, 'initializationOptions'> & {
        initializationOptions?: InitializationOptions;
    };
    runtimeEnv: ServerRuntimeEnvironment;
    onDidChangeWatchedFiles: vscode.Connection['onDidChangeWatchedFiles'];
    onDidChangeConfiguration: vscode.Connection['onDidChangeConfiguration'];
    getConfiguration<T>(section: string, scopeUri?: string): Promise<T | undefined>;
    workspaceFolders: UriMap<boolean>;
    documents: TextDocuments<SnapshotDocument>;
    reloadDiagnostics(): void;
    updateDiagnosticsAndSemanticTokens(): void;
}
export interface ServerOptions {
    watchFileExtensions?: string[];
    getServicePlugins(): LanguageServicePlugin[] | Promise<LanguageServicePlugin[]>;
    getLanguagePlugins(serviceEnv: ServiceEnvironment, projectContext: ProjectContext): LanguagePlugin[] | Promise<LanguagePlugin[]>;
    getLanguageId(uri: string): string;
}
export declare function createServerBase(connection: vscode.Connection, getRuntimeEnv: (params: ServerContext['initializeParams']) => ServerRuntimeEnvironment): {
    initialize: (params: ServerContext['initializeParams'], projectProviderFactory: ServerProjectProviderFactory, _serverOptions: ServerOptions) => Promise<vscode.InitializeResult<any>>;
    initialized: () => void;
    shutdown: () => void;
    readonly projects: ServerProjectProvider;
    readonly env: ServerRuntimeEnvironment;
};
