"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fileNameToUri = exports.uriToFileName = void 0;
const vscode_uri_1 = require("vscode-uri");
const encodeds = new Map();
function uriToFileName(uri) {
    const parsed = vscode_uri_1.URI.parse(uri);
    if (parsed.scheme === 'file') {
        return parsed.fsPath.replace(/\\/g, '/');
    }
    const encoded = encodeURIComponent(`${parsed.scheme}://${parsed.authority}`);
    encodeds.set(encoded, parsed);
    return `/${encoded}${parsed.path}`;
}
exports.uriToFileName = uriToFileName;
function fileNameToUri(fileName) {
    for (const [encoded, uri] of encodeds) {
        const prefix = `/${encoded}`;
        if (fileName.startsWith(prefix)) {
            return vscode_uri_1.URI.from({
                scheme: uri.scheme,
                authority: uri.authority,
                path: fileName.substring(prefix.length),
            }).toString();
        }
    }
    return vscode_uri_1.URI.file(fileName).toString();
}
exports.fileNameToUri = fileNameToUri;
//# sourceMappingURL=uri.js.map