"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLanguageService = void 0;
let currentProjectVersion = -1;
let currentFileName = '';
let currentSnapshot;
let languageService;
let syntaxOnlyLanguageService;
const host = {
    getProjectVersion: () => currentProjectVersion.toString(),
    getScriptFileNames: () => [currentFileName],
    getScriptVersion: () => currentProjectVersion.toString(),
    getScriptSnapshot: fileName => fileName === currentFileName ? currentSnapshot : undefined,
    getCompilationSettings: () => ({}),
    getCurrentDirectory: () => '',
    getDefaultLibFileName: () => '',
    readFile: () => undefined,
    fileExists: fileName => fileName === currentFileName,
};
function getLanguageService(ts, snapshot, languageId, syntaxOnly) {
    if (currentSnapshot !== snapshot) {
        currentSnapshot = snapshot;
        currentFileName = '/tmp.' + (languageId === 'javascript' ? 'js' :
            languageId === 'typescriptreact' ? 'tsx' :
                languageId === 'javascriptreact' ? 'jsx' :
                    'ts');
        currentProjectVersion++;
    }
    if (syntaxOnly) {
        syntaxOnlyLanguageService ??= ts.createLanguageService(host, undefined, ts.LanguageServiceMode.Syntactic);
        return {
            languageService: syntaxOnlyLanguageService,
            fileName: currentFileName,
        };
    }
    else {
        languageService ??= ts.createLanguageService(host);
        return {
            languageService,
            fileName: currentFileName,
        };
    }
}
exports.getLanguageService = getLanguageService;
//# sourceMappingURL=syntacticLanguageService.js.map